@@ -6,31 +6,32 @@ OC.L10N.register(
     "Copy link" : "Kopiañ al liamm",
     "Move" : "Diplasañ",
     "Select all" : "Dibab pep tra",
+    "Sort by created date" : "Renkañ dre deizat krouidigezh",
     "New" : "Nevez",
-    "New folder" : "Heuliad nevez",
+    "New folder" : "Teuliad nevez",
     "Search" : "Klask",
     "Empty trash bin" : "Pod-lastez goullo",
-    "Sort by created date" : "Renkañ dre deizat krouidigezh",
+    "_%n folder_::_%n folders_" : ["%nteuliad","%n deuliad","teuliadoù zo","an darn vrasañ eus an teuliadoù","%n teuliad"],
     "Submit" : "Kinnig",
-    "Cancel" : "Arrest",
+    "Cancel" : "Nullañ",
     "Recent" : "Nevez",
     "Shared with you" : "Rannet ganeoc'h",
     "Files" : "Restroù",
     "Broken links" : "Liammoù toret",
     "Delete" : "Dilemel",
-    "Settings" : "Arventennoù",
     "Need help?" : "Sikour o po ?",
+    "Created on {date}" : "Krouet da  {date}",
     "Title" : "Titl",
     "Link" : "Liamm",
     "Tags" : "Klavioù",
     "Notes" : "Notennoù",
     "Download file" : "Pelkargañ restr",
+    "Link copied" : "Liamm eilet",
     "Owner" : "Perc'henner",
     "Sharing" : "Rannan",
     "Share link" : "Rannan liamm",
     "Allow editing" : "Cheñchamentoù aotreet",
     "Allow resharing" : "Aotrea an adrannañ",
-    "Link copied" : "Liamm eilet",
     "Privacy" : "Prevezder",
     "Folder" : "Teuliad",
     "Save" : "Enrollañ"
