@@ -11,6 +11,7 @@ OC.L10N.register(
     "New folder" : "Carpeta nueva",
     "Search" : "Buscar",
     "Empty trash bin" : "Vaciar la papelera de reciclaje",
+    "_%n folder_::_%n folders_" : ["%n carpeta","%n carpetas","%n carpetas"],
     "Choose folder" : "Seleccioanr carpeta",
     "Submit" : "Enviar",
     "Cancel" : "Cancelar",
@@ -18,7 +19,6 @@ OC.L10N.register(
     "Shared with you" : "Compartido con usted",
     "Files" : "Archivos",
     "Delete" : "Borrar",
-    "Settings" : "Configuraciones ",
     "Bookmarklet" : "Marcador a código",
     "Need help?" : "¿Necesitas ayuda?",
     "Title" : "Título",
